package com.youkeda.ykd_audio_recorder_example;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.RequiresApi;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;

import android.util.Log;
import android.widget.Toast;

import com.youkeda.ykd_audio_recorder_example.interfinace.IAudioCallback;
import com.youkeda.ykd_audio_recorder_example.service.UploadingService;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import io.flutter.app.FlutterActivity;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugins.GeneratedPluginRegistrant;

import static com.youkeda.ykd_audio_recorder_example.AudioRecorder.BUFFER_SIZE;

public class MainActivity extends FlutterActivity implements IAudioCallback{











  private static final String CHANNEL = "ykd_audio_recorder";
  MethodChannel methodChannel;

  private AudioRecorder audioRecorder;
  private boolean isKeepTime;
  /**
   * 支持定时和周期性执行的线程池
   */
  private ScheduledExecutorService scheduledThreadPool = Executors.newScheduledThreadPool(1);
  private int time;
  private static final int INITIAL_DELAY = 0;
  private static final int PERIOD = 1000;

  // 声明一个集合，在后面的代码中用来存储用户拒绝授权的权
  private List<String> mPermissionList = new ArrayList<>();
  private final static int ACCESS_FINE_ERROR_CODE = 0x0245;

  private final static int HANDLER_CODE = 0x0249;
  Object mLock;

  @RequiresApi(api = Build.VERSION_CODES.N)
  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    GeneratedPluginRegistrant.registerWith(this);

    initData();

    initChannel();
  }


  /**
   * 初始化数据
   */
  private void initData() {
    audioRecorder = AudioRecorder.getInstance((IAudioCallback) this);
    scheduledThreadPool.scheduleAtFixedRate(() -> {
      if (isKeepTime) {
        ++time;
      }
    }, INITIAL_DELAY, PERIOD, TimeUnit.MILLISECONDS);

    setPermissions(new String[]{
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.READ_PHONE_STATE,
                    Manifest.permission.WRITE_SETTINGS,
                    Manifest.permission.RECORD_AUDIO},
            ACCESS_FINE_ERROR_CODE);
  }


  /**
   * 权限
   */
  private void setPermissions(String[] permissions, int permissionsCode) {
    mPermissionList.clear();
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
      for (String permission : permissions) {
        if (ContextCompat.checkSelfPermission(this, permission) != PackageManager.PERMISSION_GRANTED) {
          mPermissionList.add(permission);
        }
      }

      //未授予的权限为空，表示都授予了
      if (mPermissionList.isEmpty()) {
        showToast("已经授权");
      } else {
        //将List转为数组
        permissions = mPermissionList.toArray(new String[mPermissionList.size()]);
        ActivityCompat.requestPermissions(this, permissions, permissionsCode);
      }
    }
  }
  protected void showToast(String toastInfo) {
    Toast.makeText(this, toastInfo, Toast.LENGTH_LONG).show();
  }


  @Override
  public void showPlay(String filePath) {
    File file = new File(filePath);
    if (file.exists()) {
//            //合成完后的操作，根据需要去做处理，此处用于测试播放
//            audioRecorder.play(filePath);
      Intent intent = new Intent(this, UploadingService.class);
      intent.putExtra("test", "test");
      startService(intent);
    }
  }


  @RequiresApi(api = Build.VERSION_CODES.N)
  private void initChannel() {
    Map<String,Object> startMap=new HashMap<>();
    this.methodChannel = new MethodChannel(getFlutterView(), CHANNEL);
    this.methodChannel.setMethodCallHandler(
            (call, result) -> {
             if(call.method.equals("start")){
               try {
                 if (audioRecorder.getStatus() == AudioStatus.STATUS_NO_READY) {
                   //初始化录音
                   String fileName = new SimpleDateFormat("yyyyMMddhhmmss", Locale.CHINA).format(new Date());
                   if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                     audioRecorder.createDefaultAudio(fileName);
                   }
                   audioRecorder.startRecord();
                   isKeepTime = true;

                 } else {
                   if (audioRecorder.getStatus() == AudioStatus.STATUS_START) {
                     phoneToPause();
                   } else {
                     audioRecorder.startRecord();
                     isKeepTime = true;
                   }
                 }
                 startMap.put("success",true);
                 result.success(startMap);
               } catch (IllegalStateException e) {
                 e.printStackTrace();
                 startMap.put("error",e.toString());
                 result.success(startMap  );
               }
             }else  if(call.method.equals("stop")){
               Map<String,Object> stringObjectMap=  finishAndReset();
                result.success(stringObjectMap);

             }else if(call.method.equals("pause")){

               phoneToPause();


             }else  if(call.method.equals("getVolumn")){

               new Thread(new Runnable() {
                 @Override
                 public void run() {
                  audioRecorder.getNoiseLevel();
                   result.success(audioRecorder.volume);
                   System.out.println(audioRecorder.volume+"哈哈哈哈哈");
                 }
               }).start();

             }else if(call.method.equals("resume")){
               try {
                 if (audioRecorder.getStatus() == AudioStatus.STATUS_NO_READY) {
                   //初始化录音
                   String fileName = new SimpleDateFormat("yyyyMMddhhmmss", Locale.CHINA).format(new Date());
                   if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                     audioRecorder.createDefaultAudio(fileName);
                   }
                   audioRecorder.startRecord();
                   isKeepTime = true;

                 } else {
                   if (audioRecorder.getStatus() == AudioStatus.STATUS_START) {
                     phoneToPause();
                   } else {
                     audioRecorder.startRecord();
                     isKeepTime = true;
                   }
                 }
                 startMap.put("success",true);
                 result.success(startMap);
               } catch (IllegalStateException e) {
                 e.printStackTrace();
                 startMap.put("error",e.toString());
                 result.success(startMap  );
               }
             }

            });
  }
  @RequiresApi(api = Build.VERSION_CODES.N)
  private Map<String,Object> finishAndReset() {
    isKeepTime = false;
    Map<String,Object> stringObjectMap= audioRecorder.stopRecord();
    time = 0;
    return  stringObjectMap;
  }
  /**
   * 暂停录音和状态修改
   */
  @RequiresApi(api = Build.VERSION_CODES.N)
  private void phoneToPause() {
   audioRecorder.pauseRecord();
    isKeepTime = false;
  }

}
