#import "YkdAudioRecorderPlugin.h"
#import <AVFoundation/AVFoundation.h>

@interface YkdAudioRecorderPlugin()<AVAudioRecorderDelegate>

@property (assign, nonatomic) bool isRecording;
@property (assign, nonatomic) bool hasPermission;
@property (strong, nonatomic) NSString *mExtension;
@property (strong, nonatomic) NSString *mPath;
@property (strong, nonatomic) NSDate *startTime;
@property (strong, nonatomic) AVAudioRecorder *audioRecorder;


@end

@implementation YkdAudioRecorderPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"ykd_audio_recorder"
            binaryMessenger:[registrar messenger]];
  YkdAudioRecorderPlugin* instance = [[YkdAudioRecorderPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"start" isEqualToString:call.method]) {
      NSDictionary *param = @{};
      self.startTime = [NSDate new];
      if(call.arguments != nil){
          param = call.arguments;
          self.mExtension = param[@"extension"] != nil ? param[@"extension"] : @"";
          self.mPath = param[@"path"] != nil ? param[@"path"] : @"";
      }
      if(self.mPath == nil || [self.mPath isEqualToString:@""]){
          NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
          NSTimeInterval time = [self.startTime timeIntervalSince1970];
          self.mPath = [NSString stringWithFormat:@"%@/%f.m4a", [paths objectAtIndex:0], time];
      }
      NSMutableDictionary * recordSetting = [[NSMutableDictionary alloc]init];
      //设置录音格式
      [recordSetting  setValue:[NSNumber numberWithInt:kAudioFormatMPEG4AAC] forKey:AVFormatIDKey];
      //设置录音采样率（HZ）
      [recordSetting setValue:[NSNumber numberWithFloat:4000] forKey:AVSampleRateKey];
      //录音通道数
      [recordSetting setValue:[NSNumber  numberWithInt:1] forKey:AVNumberOfChannelsKey];
      //线性采样位数
      [recordSetting  setValue:[NSNumber numberWithInt:8] forKey:AVLinearPCMBitDepthKey];
      //录音的质量
      [recordSetting  setValue:[NSNumber numberWithInt:AVAudioQualityHigh] forKey:AVEncoderAudioQualityKey];
      //初始化AVAudioRecorder
      AVAudioSession *session = [AVAudioSession sharedInstance];
      [session setCategory:AVAudioSessionCategoryPlayAndRecord error:nil];
      [session setActive: YES error: nil];
      
      NSError * error ;
      self.audioRecorder = [[AVAudioRecorder alloc]initWithURL:[NSURL URLWithString:self.mPath] settings:recordSetting error:&error];
      //开启音量监测
      self.audioRecorder.meteringEnabled = YES;
      self.audioRecorder.delegate = self;
      [[NSNotificationCenter defaultCenter] addObserver:self
                                               selector:@selector(audioSessionWasInterrupted:)
                                                   name:AVAudioSessionInterruptionNotification
                                                 object:self.audioRecorder];
      if(error){
          NSLog(@"创建录音对象时发生错误，错误信息：%@",error.localizedDescription);
          result(@{@"success": @(NO), @"error": error.localizedDescription});
          return;
      }
      bool success = [self.audioRecorder prepareToRecord];
      if(!success){
          result(@{@"success": @(success)});
          return;
      }
      [self.audioRecorder record];
      result(@{@"success": @(YES)});
  } else if([@"stop" isEqualToString:call.method]){
      int second = (int)self.audioRecorder.currentTime;
      [self.audioRecorder stop];
      NSString *tmpPath = self.mPath;
      AVAudioSession *audioSession = [AVAudioSession sharedInstance];
      [audioSession setCategory:AVAudioSessionCategoryPlayAndRecord  withOptions:AVAudioSessionCategoryOptionDefaultToSpeaker error:nil ];
      [audioSession setActive:NO error:nil];
      result(@{
               @"duration": @(second),
               @"path": tmpPath
               });
      self.mPath = nil;
  } else if([@"pause" isEqualToString:call.method]){
      [self.audioRecorder pause];
  } else if([@"resume" isEqualToString:call.method]){
      [self.audioRecorder record];
  } else if([@"getVolumn" isEqualToString:call.method]){
      [self.audioRecorder updateMeters];
      float level;
      float minDecibels = -80.0f;
      float decibels = [self.audioRecorder averagePowerForChannel:0];
      if (decibels < minDecibels){
          level = 0.0f;
      }
      else if (decibels >= 0.0f){
          level = 1.0f;
      } else {
          float root = 2.0f;
          float minAmp = powf(10.0f, 0.05f * minDecibels);
          float inverseAmpRange = 1.0f / (1.0f - minAmp);
          float amp = powf(10.0f, 0.05f * decibels);
          float adjAmp = (amp - minAmp) * inverseAmpRange;
          
          level = powf(adjAmp, 1.0f / root);
      }
      result(@(level));
  }
}


- (void)audioSessionWasInterrupted:(NSNotification *)notification{
    if (AVAudioSessionInterruptionTypeBegan == [notification.userInfo[AVAudioSessionInterruptionTypeKey] intValue]){
        [self.audioRecorder pause];
    }
    else if (AVAudioSessionInterruptionTypeEnded == [notification.userInfo[AVAudioSessionInterruptionTypeKey] intValue]){
        [self.audioRecorder record];
    }
}

@end
