import 'dart:async';
import 'dart:io';

import 'package:file/local.dart';
import 'package:flutter/services.dart';
import 'package:path/path.dart' as p;

class YkdAudioRecorder {
  static const MethodChannel _channel =
      const MethodChannel('ykd_audio_recorder');

  static LocalFileSystem fs = LocalFileSystem();

  static Future<RecordStatus> start({String path}) async {
    if (path != null) {
      File file = fs.file(path);
      if (await file.exists()) {
        throw new Exception("A file already exists at the path :" + path);
      } else if (!await file.parent.exists()) {
        throw new Exception("The specified parent directory does not exist");
      }
    }
    Map<String, Object> map = Map<String, Object>.from(
        await _channel.invokeMethod('start', {'path': path}));
    return new RecordStatus(success: map['success'], error: map['error']);
  }

  static Future<RecordInfo> stop() async {
    Map<String, Object> map =
        Map<String, Object>.from(await _channel.invokeMethod('stop'));
    return new RecordInfo(
        duration: new Duration(seconds: map['duration']), path: map['path']);
  }

  static Future pause() async {
    return await _channel.invokeMethod('pause');
  }

  static Future<double> getVolumn() async {
    return await _channel.invokeMethod('getVolumn');
  }

  static Future resume() async {
    return await _channel.invokeMethod('resume');
  }
}

class RecordStatus {
  bool success;
  String error;
  RecordStatus({this.success, this.error});
}

class RecordInfo {
  String path;
  Duration duration;
  RecordInfo({this.duration, this.path});
}
